@@ -1,17 +1,20 @@
-import re
-import glob
-import os
-from os.path import join, expanduser
-from gettext import gettext as _
+from deskbar.core.Utils import get_xdg_data_dirs, is_program_in_path, spawn_async, is_executable, PATH
 from deskbar.defs import VERSION
+from deskbar.handlers.actions.OpenDesktopFileAction import OpenDesktopFileAction, parse_desktop_file, parse_desktop_filename
+from deskbar.handlers.actions.OpenWithApplicationAction import OpenWithApplicationAction
+from gettext import gettext as _
+from os.path import join, expanduser, isdir
+import deskbar, deskbar.core.Indexer
+import deskbar.core.Utils
+import deskbar.core.gnomedesktop
+import deskbar.interfaces.Action
+import deskbar.interfaces.Match
+import deskbar.interfaces.Module
+import glob
 import gobject
 import gtk
-import deskbar, deskbar.core.Indexer, deskbar.core.Utils
-import deskbar.interfaces.Module, deskbar.interfaces.Match, deskbar.core.gnomedesktop
-from deskbar.core.Utils import get_xdg_data_dirs, is_program_in_path, spawn_async
-from deskbar.handlers.actions.OpenWithApplicationAction import OpenWithApplicationAction
-from deskbar.handlers.actions.OpenDesktopFileAction import OpenDesktopFileAction, parse_desktop_file, parse_desktop_filename
-import deskbar.interfaces.Action
+import os
+import re
 
 HANDLERS = [
     "ProgramsHandler",
@@ -21,6 +24,7 @@ HANDLERS = [
 
 EXACT_MATCH_PRIO = 50
 EXACT_WORD_PRIO = 5
+DESKTOP_FILE_PRIO = 25
 
 class GenericAction(OpenWithApplicationAction):
     
@@ -69,6 +73,9 @@ class GenericProgramMatch(deskbar.interfaces.Match):
 
     def get_hash(self):
         return "generic_"+self._display_prog
+    
+    def get_program(self):
+        return self._display_prog
         
 class GnomeDictMatch(GenericProgramMatch):
     def __init__(self, **args):
@@ -174,7 +181,10 @@ class DevhelpHandler(SpecialProgramHandler):
              'version': VERSION}
     
     def __init__(self):
-        SpecialProgramHandler.__init__(self, "devhelp.desktop")
+        if SpecialProgramHandler.desktop_file_exists("devhelp.desktop"):
+        	SpecialProgramHandler.__init__(self, "devhelp.desktop")
+        else:
+      		SpecialProgramHandler.__init__(self, "gnome-devhelp.desktop")
     
     def create_match(self, desktop, f):
         return DevhelpMatch(
@@ -185,7 +195,7 @@ class DevhelpHandler(SpecialProgramHandler):
         
     @staticmethod
     def has_requirements():
-        if not SpecialProgramHandler.desktop_file_exists("devhelp.desktop"):
+        if not (SpecialProgramHandler.desktop_file_exists("devhelp.desktop") or SpecialProgramHandler.desktop_file_exists("gnome-devhelp.desktop")):
             DevhelpHandler.INSTRUCTIONS = _("Devhelp is not installed")
             return False
         return True
@@ -205,12 +215,12 @@ class OpenPathProgramAction(deskbar.interfaces.Action):
                 import subprocess
 
                 prog = subprocess.Popen(
-                    text.split(" "),
+                    self._name.split(" "),
                     stdout=subprocess.PIPE,
                     stderr=subprocess.STDOUT)
                 
                 zenity = subprocess.Popen(
-                    ["zenity", "--title="+text,
+                    ["zenity", "--title="+self._name,
                         "--window-icon="+join(deskbar.ART_DATA_DIR, "generic.png"),
                         "--width=700",
                         "--height=500",
@@ -225,7 +235,7 @@ class OpenPathProgramAction(deskbar.interfaces.Action):
                 #No zenity, get out of the if, and launch without GUI
                 pass
         
-        spawn_async(text.split(" "))
+        spawn_async(self._name.split(" "))
         
     def get_hash(self):
         if self.use_terminal:
@@ -235,9 +245,9 @@ class OpenPathProgramAction(deskbar.interfaces.Action):
 
     def get_verb(self):
         if self.use_terminal:
-            return _("Execute %s in terminal") % "<b>%(text)s</b>"
+            return _("Execute %s in terminal") % "<b>%(name)s</b>"
         else:
-            return _("Execute %s") % "<b>%(text)s</b>"
+            return _("Execute %s") % "<b>%(name)s</b>"
 
 class PathProgramMatch(deskbar.interfaces.Match):
     
@@ -247,8 +257,19 @@ class PathProgramMatch(deskbar.interfaces.Match):
         self.add_action( OpenPathProgramAction(command, False), True )
         self.add_action( OpenPathProgramAction(command, True) )
         
-    def get_hash(self, text=None):
-        return text
+    def get_hash(self):
+        return self._name
+   
+class StartsWithPathProgramMatch(deskbar.interfaces.Match):
+    
+    def __init__(self, command, priority=0, **args):
+        deskbar.interfaces.Match.__init__(self, name=command, icon="gtk-execute", category="actions", **args)
+        self._command = command
+        self.add_action( OpenPathProgramAction(command, False), True )
+        self.add_action( OpenPathProgramAction(command, True) )
+        
+    def get_hash(self):
+        return self._command
         
 class ProgramsHandler(deskbar.interfaces.Module):
     
@@ -265,22 +286,50 @@ class ProgramsHandler(deskbar.interfaces.Module):
         self._scan_desktop_files()
         
     def query(self, query):
-        result = self.query_path_programs(query)
-        result += self.query_desktop_programs(query)
-        self.set_priority_for_matches(result)
+        result = self.query_desktop_programs(query)
+        
+        desktop_progs = set()
+        for match in result:
+            desktop_progs.add( match.get_program () )
+            match.set_priority (self.get_priority() + DESKTOP_FILE_PRIO + match.get_priority())
+        
+        path_result = self.query_path_programs(query, desktop_progs)
+        result += path_result
+        
         self._emit_query_ready(query, result )
         
-    def query_path_programs(self, query):
-        program = query.split(" ")[0]
-        if is_program_in_path(program):
-            return [PathProgramMatch(program, query)]
+    def query_path_programs(self, query, desktop_progs):
+        """
+        @param query: Query string
+        @param desktop_progs: Names of binaries from .desktop files
+        """
+        args = query.split(" ")     
+        program = args[0]
+        
+        if len(args) == 1:
+            results = []
+            for pathdir in PATH:
+                for f in os.listdir(pathdir):
+                    pathprog = join(pathdir, f)
+                    if (not (f in desktop_progs)) and not isdir(pathprog) \
+                    and f.lower().startswith(program) and is_executable(pathprog):
+                        match = StartsWithPathProgramMatch(f)
+                        match.set_priority (self.get_priority() + get_priority_for_name(query, f))
+                        results.append( match )
+            return results
         else:
-            return []
+            # We have arguments, execute the command as typed in by the user
+            if not (program in desktop_progs) and is_program_in_path(program):
+                match = PathProgramMatch(program, query)
+                match.set_priority (self.get_priority() + EXACT_MATCH_PRIO)
+                return [match]
+            else:
+                return []
     
     def query_desktop_programs(self, query):
         result = []
         for match in self._indexer.look_up(query):
-            match._priority = get_priority_for_name(query, match._desktop.get_string("Exec"))
+            match.set_priority (get_priority_for_name(query, match._desktop.get_string("Exec")))
             result.append(match)
         return result
                 
@@ -304,7 +353,8 @@ class ProgramsHandler(deskbar.interfaces.Module):
                                 ), match)
 
 def get_priority_for_name(query, name):
-    if name.split(" ")[0].endswith(query):
+    bin = name.split(" ")[0]
+    if bin == query:
         return EXACT_MATCH_PRIO
     else:
         return EXACT_WORD_PRIO
