@@ -84,11 +84,11 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         hhbox.pack_start(self.hview)
         hlabel.set_mnemonic_widget(self.hview)
         
-        empty_button = gtk.Button()
-        empty_button.set_image( gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU) )
-        empty_button.connect("clicked", self._controller.on_clear_history)
-        empty_button.show()
-        hhbox.pack_start(empty_button, False)
+        self.empty_button = gtk.Button()
+        self.empty_button.set_image( gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU) )
+        self.empty_button.connect("clicked", self._controller.on_clear_history)
+        self.empty_button.show()
+        hhbox.pack_start(self.empty_button, False)
         
         # Results TreeView
         self.treeview_model = CuemiacModel ()
@@ -150,9 +150,6 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         self.results_box.pack_start(self.actions_box)
         self.vbox_main.pack_start(self.results_box)
    
-    def __on_row_act(self, view, path, column):
-        print "======================"
-    
     def clear_all(self):
         deskbar.interfaces.View.clear_all(self)
         width, height = self.get_size()
@@ -190,7 +187,11 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         self.window.set_user_time(time)
         self.present()
         self.move( self._model.get_window_x(), self._model.get_window_y() )
-    
+        
+    def mark_history_empty(self, val):
+        self.hview.set_sensitive (not val)
+        self.empty_button.set_sensitive (not val)
+        
     def __show_matches(self):
         self.scrolled_results.show()
         self.actions_box.hide()
@@ -284,4 +285,4 @@ class CuemiacWindowView(deskbar.interfaces.View, gtk.Window):
         """
         Save window height before results_box disappears
         """
-        self._model.set_window_height( self.get_size()[1] )
\ No newline at end of file
+        self._model.set_window_height( self.get_size()[1] )
